﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;
using System.ServiceModel;
using System.Threading;
using System.Security;
using HeuristicLab.Hive.Server.DataAccess;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;

namespace HeuristicLab.Hive.Server.Core.Authorization {
  public class AuthorizationManager : IAuthorizationManager {
    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    public void AuthorizeForJobs(params Guid[] jobIds) {
      if (!(Identity.IsAuthenticated && IsAuthorizedForJobs(jobIds))) {
        throw new SecurityException("User '" + Identity.Name + "' is not authorized to access job (Id: " + string.Join(", ", jobIds.Select(x => x.ToString()).ToArray()) + ")");
      }
    }

    public string UserId {
      get { return Identity.Name; }
    }
    
    private bool IsAuthorizedForJobs(params Guid[] jobIds) {
      return DaoLocator.JobDao.IsUserAuthorizedForJobs(Identity.Name, jobIds);
    }

    private IIdentity Identity {
      get { return ServiceSecurityContext.Current.PrimaryIdentity; }
    }
  }
}
