﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;

namespace HeuristicLab.Hive.Server.LINQDataAccess {
  public class SlaveConfigDao: BaseDao<SlaveConfigDto, SlaveConfig>, ISlaveConfigDao {

    #region IGenericDao<SlaveConfigDto,SlaveConfig> Members

    public SlaveConfigDto FindById(Guid id) {
      return (from cc in Context.SlaveConfigs
              where cc.SlaveConfigId.Equals(id)
              select EntityToDto(cc, null)).SingleOrDefault();
    }

    public IEnumerable<SlaveConfigDto> FindAll() {
      return (from cc in Context.SlaveConfigs              
              select EntityToDto(cc, null)).ToList();
    }

    public SlaveConfigDto Insert(SlaveConfigDto bObj) {
      SlaveConfig c = DtoToEntity(bObj, null);
      Context.SlaveConfigs.InsertOnSubmit(c);
      CommitChanges();
      bObj.Id = c.SlaveConfigId;
      return bObj;  
    }

    public void Delete(SlaveConfigDto bObj) {
      Context.SlaveConfigs.DeleteOnSubmit(Context.SlaveConfigs.SingleOrDefault(c => c.SlaveConfigId.Equals(bObj.Id)));
      CommitChanges();
    }

    public void Update(SlaveConfigDto bObj) {
      SlaveConfig cc = Context.SlaveConfigs.SingleOrDefault(c => c.SlaveConfigId.Equals(bObj.Id));
      DtoToEntity(bObj, cc);
      CommitChanges();
    }

    #endregion

    public override SlaveConfig DtoToEntity(SlaveConfigDto source, SlaveConfig target) {
      if (source == null)
        return null;
      if (target == null)
        target = new SlaveConfig();

      target.SlaveConfigId = source.Id;
      target.HeartBeatIntervall = source.HeartBeatIntervall;
      
      return target;
    }

    public override SlaveConfigDto EntityToDto(SlaveConfig source, SlaveConfigDto target) {
      if (source == null)
        return null;
      if (target == null)
        target = new SlaveConfigDto();

      target.Id = source.SlaveConfigId;
      target.HeartBeatIntervall = source.HeartBeatIntervall;

      return target;

    }
  }
}
